/* Copyright (c) 2016
 * The Trustees of Columbia University in the City of New York
 * All rights reserved.
 *
 * Authors:  Orestis Polychroniou  (orestis@cs.columbia.edu)
 *           Eva Sitaridi (eva@cs.columbia.edu)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <assert.h>


static void connect(int32_t *states,
                    int32_t  src,
                    int32_t  dst,
                 const char *str)
{
	size_t i = src;
	states += i << 8;
	for (i = 0 ; str[i] ; ++i) {
		uint8_t x = str[i + 0];
		uint8_t y = str[i + 1];
		if (x != '-' || !i || !y) y = x;
		else {
			x = str[i++ - 1] + 1;
			assert(x <= y);
		}
		do {
			assert(isprint(x));
			assert(states[x] == -1);
			states[x] = dst;
		} while (x++ != y);
	}
}

enum {init = 0, user, at, dotatom, dot, tld1, tld2, tld3, tld4};

const char *state_name[] = {"init", "user", "at", "dotatom", "dot",
                            "tld1", "tld2", "tld3", "tld4"};

const char *dfa_state(size_t s)
{
	size_t n = sizeof(state_name) / sizeof(char*);
	return s >= n ? NULL : state_name[s];
}

int32_t *dfa(size_t *ini_p, size_t *rej_p, size_t *acc_p)
{
	size_t ini = 0, rej = 6, acc = 3;
	int32_t *states = malloc(((acc + rej) << 10) + 4);
	memset(states, -1, (acc + rej) << 10);
	connect(states, init, user, "A-Za-z0-9.+_%-");
	connect(states, user, user, "A-Za-z0-9.+_%-");
	connect(states, user, at, "@");
	connect(states, at, dotatom, "A-Za-z0-9-");
	connect(states, dotatom, dotatom, "A-Za-z0-9-");
	connect(states, dotatom, dot, ".");
	connect(states, dot, dotatom, "0-9-");
	connect(states, dot, tld1, "A-Za-z");
	connect(states, tld1, dotatom, "0-9-");
	connect(states, tld1, dot, ".");
	connect(states, tld1, tld2, "A-Za-z");
	connect(states, tld2, dotatom, "0-9-");
	connect(states, tld2, dot, ".");
	connect(states, tld2, tld3, "A-Za-z");
	connect(states, tld3, dotatom, "0-9-");
	connect(states, tld3, dot, ".");
	connect(states, tld3, tld4, "A-Za-z");
	connect(states, tld4, dotatom, "A-Za-z0-9-");
	connect(states, tld4, dot, ".");
	*ini_p = ini;
	*rej_p = rej;
	*acc_p = acc;
	return states;
}
