/* Copyright (c) 2016
 * The Trustees of Columbia University in the City of New York
 * All rights reserved.
 *
 * Authors:  Orestis Polychroniou  (orestis@cs.columbia.edu)
 *           Eva Sitaridi (eva@cs.columbia.edu)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <assert.h>


enum {
	S1 = 0,
	S2,
	S3,
	S_f,
	S_h,
	S_ht,
	S_Xt,
	S_Xtp,
	S_Xtps,
	UI1,
	UI1_1,
	UI1_2,
	UI1_25,
	UI1_XY,
	UI1_XYZ,
	UI2,
	UI2_1,
	UI2_2,
	UI2_25,
	UI2_XY,
	UI2_XYZ,
	UI3,
	UI3_1,
	UI3_2,
	UI3_25,
	UI3_XY,
	UI3_XYZ,
	UI4,
	UH1,
	UH2,
	UH3,
	UT0,
	UT1,
	U,
	I1,
	I1_1,
	I1_2,
	I1_25,
	I1_XY,
	I1_XYZ,
	I2,
	I2_1,
	I2_2,
	I2_25,
	I2_XY,
	I2_XYZ,
	I3,
	I3_1,
	I3_2,
	I3_25,
	I3_XY,
	I3_XYZ,
	I4,
	H1,
	H2,
	H3,
	T0,
	T1,
	A3,
	A4,
	Q2,
	Q3,
	F2,
	F3,
	UI4_1,
	UI4_2,
	UI4_25,
	UI4_XY,
	UI4_XYZ,
	UT2,
	UT3,
	UT4,
	UT5,
	UT6,
	UP,
	I4_1,
	I4_2,
	I4_25,
	I4_XY,
	I4_XYZ,
	T2,
	T3,
	T4,
	T5,
	T6,
	P,
	A1,
	A2,
	Q1,
	F1};

static void connect(int32_t *states,
                    int32_t  src,
                    int32_t  dst,
                 const char *str)
{
	size_t i = src;
	states += i << 8;
	for (i = 0 ; str[i] ; ++i) {
		uint8_t x = str[i + 0];
		uint8_t y = str[i + 1];
		if (x != '-' || !i || !y) y = x;
		else {
			x = str[i++ - 1] + 1;
			assert(x <= y);
		}
		do {
			assert(isprint(x));
			assert(states[x] == -1);
			states[x] = dst;
		} while (x++ != y);
	}
}

const char *dfa_state(size_t s) { return NULL; }

int32_t *dfa(size_t *ini_p, size_t *rej_p, size_t *acc_p)
{
	size_t acc_rej = 90;
	int32_t *states = malloc(acc_rej << 10);
	memset(states, -1, acc_rej << 10);
	*ini_p =  0;
	*rej_p = 64;
	*acc_p = 26;
	connect(states, S1, S_f, "f");
	connect(states, S1, S_h, "h");
	connect(states, S_h, S_ht, "t");
	connect(states, S_f, S_Xt, "t");
	connect(states, S_ht, S_Xt, "t");
	connect(states, S_Xt, S_Xtp, "p");
	connect(states, S_Xtp, S_Xtps, "s");
	connect(states, S_Xtp, S2, ":");
	connect(states, S_Xtps, S2, ":");
	connect(states, S2, S3, "/");
	connect(states, S3, UI1, "/");
	connect(states, UI1, UI1_XYZ, "0");
	connect(states, UI1, UI1_1, "1");
	connect(states, UI1, UI1_2, "2");
	connect(states, UI1, UI1_XY, "3-9");
	connect(states, UI1_1, UI1_XY, "0-9");
	connect(states, UI1_2, UI1_XY, "0-4");
	connect(states, UI1_2, UI1_25, "5");
	connect(states, UI1_25, UI1_XYZ, "0-5");
	connect(states, UI1_XY, UI1_XYZ, "0-9");
	connect(states, UI1_1, UI2, ".");
	connect(states, UI1_2, UI2, ".");
	connect(states, UI1_25, UI2, ".");
	connect(states, UI1_XY, UI2, ".");
	connect(states, UI1_XYZ, UI2, ".");
	connect(states, UI1, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI1_1, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI1_2, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI1_25, UH1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, UI1_XY, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI1_XYZ, UH1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, UI1, UH2, "%");
	connect(states, UI1_1, UH2, "%");
	connect(states, UI1_2, UH2, "%");
	connect(states, UI1_25, UH2, "%");
	connect(states, UI1_XY, UH2, "%");
	connect(states, UI1_XYZ, UH2, "%");
	connect(states, UI1, U, ":");
	connect(states, UI1_1, U, ":");
	connect(states, UI1_2, U, ":");
	connect(states, UI1_25, U, ":");
	connect(states, UI1_XY, U, ":");
	connect(states, UI1_XYZ, U, ":");
	connect(states, UI2, UI2_XYZ, "0");
	connect(states, UI2, UI2_1, "1");
	connect(states, UI2, UI2_2, "2");
	connect(states, UI2, UI2_XY, "3-9");
	connect(states, UI2_1, UI2_XY, "0-9");
	connect(states, UI2_2, UI2_XY, "0-4");
	connect(states, UI2_2, UI2_25, "5");
	connect(states, UI2_25, UI2_XYZ, "0-5");
	connect(states, UI2_XY, UI2_XYZ, "0-9");
	connect(states, UI2_1, UI3, ".");
	connect(states, UI2_2, UI3, ".");
	connect(states, UI2_25, UI3, ".");
	connect(states, UI2_XY, UI3, ".");
	connect(states, UI2_XYZ, UI3, ".");
	connect(states, UI2, UT1, "A-Za-z");
	connect(states, UI2, UH1, "-!$&'()*+,;=");
	connect(states, UI2_1, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI2_2, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI2_25, UH1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, UI2_XY, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI2_XYZ, UH1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, UI2, UH2, "%");
	connect(states, UI2_1, UH2, "%");
	connect(states, UI2_2, UH2, "%");
	connect(states, UI2_25, UH2, "%");
	connect(states, UI2_XY, UH2, "%");
	connect(states, UI2_XYZ, UH2, "%");
	connect(states, UI2, U, ":");
	connect(states, UI2_1, U, ":");
	connect(states, UI2_2, U, ":");
	connect(states, UI2_25, U, ":");
	connect(states, UI2_XY, U, ":");
	connect(states, UI2_XYZ, U, ":");
	connect(states, UI3, UI3_XYZ, "0");
	connect(states, UI3, UI3_1, "1");
	connect(states, UI3, UI3_2, "2");
	connect(states, UI3, UI3_XY, "3-9");
	connect(states, UI3_1, UI3_XY, "0-9");
	connect(states, UI3_2, UI3_XY, "0-4");
	connect(states, UI3_2, UI3_25, "5");
	connect(states, UI3_25, UI3_XYZ, "0-5");
	connect(states, UI3_XY, UI3_XYZ, "0-9");
	connect(states, UI3_1, UI4, ".");
	connect(states, UI3_2, UI4, ".");
	connect(states, UI3_25, UI4, ".");
	connect(states, UI3_XY, UI4, ".");
	connect(states, UI3_XYZ, UI4, ".");
	connect(states, UI3, UT1, "A-Za-z");
	connect(states, UI3, UH1, "-!$&'()*+,;=");
	connect(states, UI3_1, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI3_2, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI3_25, UH1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, UI3_XY, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI3_XYZ, UH1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, UI3, UH2, "%");
	connect(states, UI3_1, UH2, "%");
	connect(states, UI3_2, UH2, "%");
	connect(states, UI3_25, UH2, "%");
	connect(states, UI3_XY, UH2, "%");
	connect(states, UI3_XYZ, UH2, "%");
	connect(states, UI3, U, ":");
	connect(states, UI3_1, U, ":");
	connect(states, UI3_2, U, ":");
	connect(states, UI3_25, U, ":");
	connect(states, UI3_XY, U, ":");
	connect(states, UI3_XYZ, U, ":");
	connect(states, UI4, UI4_XYZ, "0");
	connect(states, UI4, UI4_1, "1");
	connect(states, UI4, UI4_2, "2");
	connect(states, UI4, UI4_XY, "3-9");
	connect(states, UI4_1, UI4_XY, "0-9");
	connect(states, UI4_2, UI4_XY, "0-4");
	connect(states, UI4_2, UI4_25, "5");
	connect(states, UI4_25, UI4_XYZ, "0-5");
	connect(states, UI4_XY, UI4_XYZ, "0-9");
	connect(states, UI4_1, UP, ":");
	connect(states, UI4_2, UP, ":");
	connect(states, UI4_25, UP, ":");
	connect(states, UI4_XY, UP, ":");
	connect(states, UI4_XYZ, UP, ":");
	connect(states, UI4_1, UT0, ".");
	connect(states, UI4_2, UT0, ".");
	connect(states, UI4_25, UT0, ".");
	connect(states, UI4_XY, UT0, ".");
	connect(states, UI4_XYZ, UT0, ".");
	connect(states, UI4, UT1, "A-Za-z");
	connect(states, UI4, UH1, "-!$&'()*+,;=");
	connect(states, UI4_1, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI4_2, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI4_25, UH1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, UI4_XY, UH1, "-!$&'()*+,;=A-Za-z");
	connect(states, UI4_XYZ, UH1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, UI4, UH2, "%");
	connect(states, UI4_1, UH2, "%");
	connect(states, UI4_2, UH2, "%");
	connect(states, UI4_25, UH2, "%");
	connect(states, UI4_XY, UH2, "%");
	connect(states, UI4_XYZ, UH2, "%");
	connect(states, UI4_1, A1, "/");
	connect(states, UI4_2, A1, "/");
	connect(states, UI4_25, A1, "/");
	connect(states, UI4_XY, A1, "/");
	connect(states, UI4_XYZ, A1, "/");
	connect(states, UI4_1, Q1, "?");
	connect(states, UI4_2, Q1, "?");
	connect(states, UI4_25, Q1, "?");
	connect(states, UI4_XY, Q1, "?");
	connect(states, UI4_XYZ, Q1, "?");
	connect(states, UI4_1, F1, "#");
	connect(states, UI4_2, F1, "#");
	connect(states, UI4_25, F1, "#");
	connect(states, UI4_XY, F1, "#");
	connect(states, UI4_XYZ, F1, "#");
	connect(states, UI4, U, ":");
	connect(states, UH1, UT0, ".");
	connect(states, UH1, UH1, "-_~A-Za-z0-9!$&'()*+,;=");
	connect(states, UH1, UH2, "%");
	connect(states, UH2, UH3, "0-9A-F");
	connect(states, UH3, UH1, "0-9A-F");
	connect(states, UH1, U, ":");
	connect(states, UH2, U, ":");
	connect(states, UH3, U, ":");
	connect(states, UT1, UT0, ".");
	connect(states, UT2, UT0, ".");
	connect(states, UT3, UT0, ".");
	connect(states, UT4, UT0, ".");
	connect(states, UT5, UT0, ".");
	connect(states, UT6, UT0, ".");
	connect(states, UT0, UT1, "A-Za-z");
	connect(states, UT1, UT2, "A-Za-z");
	connect(states, UT2, UT3, "A-Za-z");
	connect(states, UT3, UT4, "A-Za-z");
	connect(states, UT4, UT5, "A-Za-z");
	connect(states, UT5, UT6, "A-Za-z");
	connect(states, UT0, UH1, "-_~0-9!$&'()*+,;=");
	connect(states, UT1, UH1, "-_~0-9!$&'()*+,;=");
	connect(states, UT2, UH1, "-_~0-9!$&'()*+,;=");
	connect(states, UT3, UH1, "-_~0-9!$&'()*+,;=");
	connect(states, UT4, UH1, "-_~0-9!$&'()*+,;=");
	connect(states, UT5, UH1, "-_~0-9!$&'()*+,;=");
	connect(states, UT6, UH1, "-_~0-9!$&'()*+,;=A-Za-z");
	connect(states, UT0, UH2, "%");
	connect(states, UT1, UH2, "%");
	connect(states, UT2, UH2, "%");
	connect(states, UT3, UH2, "%");
	connect(states, UT4, UH2, "%");
	connect(states, UT5, UH2, "%");
	connect(states, UT6, UH2, "%");
	connect(states, UT2, A1, "/");
	connect(states, UT3, A1, "/");
	connect(states, UT4, A1, "/");
	connect(states, UT5, A1, "/");
	connect(states, UT6, A1, "/");
	connect(states, UT2, Q1, "?");
	connect(states, UT3, Q1, "?");
	connect(states, UT4, Q1, "?");
	connect(states, UT5, Q1, "?");
	connect(states, UT6, Q1, "?");
	connect(states, UT2, F1, "#");
	connect(states, UT3, F1, "#");
	connect(states, UT4, F1, "#");
	connect(states, UT5, F1, "#");
	connect(states, UT6, F1, "#");
	connect(states, UT0, U, ":");
	connect(states, UT1, U, ":");
	connect(states, UT2, UP, ":");
	connect(states, UT3, UP, ":");
	connect(states, UT4, UP, ":");
	connect(states, UT5, UP, ":");
	connect(states, UT6, UP, ":");
	connect(states, UP, UP, "0-9");
	connect(states, UP, U, "-!$&'()*+,;=A-Za-z:");
	connect(states, UP, A1, "/");
	connect(states, UP, Q1, "?");
	connect(states, UP, F1, "#");
	connect(states, UP, I1, "@");
	connect(states, U, U, "-!$&'()*+,;=A-Za-z0-9:");
	connect(states, U, I1, "@");
	connect(states, I1, I1_XYZ, "0");
	connect(states, I1, I1_1, "1");
	connect(states, I1, I1_2, "2");
	connect(states, I1, I1_XY, "3-9");
	connect(states, I1_1, I1_XY, "0-9");
	connect(states, I1_2, I1_XY, "0-4");
	connect(states, I1_2, I1_25, "5");
	connect(states, I1_25, I1_XYZ, "0-5");
	connect(states, I1_XY, I1_XYZ, "0-9");
	connect(states, I1_1, I2, ".");
	connect(states, I1_2, I2, ".");
	connect(states, I1_25, I2, ".");
	connect(states, I1_XY, I2, ".");
	connect(states, I1_XYZ, I2, ".");
	connect(states, I1, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I1_1, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I1_2, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I1_25, H1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, I1_XY, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I1_XYZ, H1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, I1, H2, "%");
	connect(states, I1_1, H2, "%");
	connect(states, I1_2, H2, "%");
	connect(states, I1_25, H2, "%");
	connect(states, I1_XY, H2, "%");
	connect(states, I1_XYZ, H2, "%");
	connect(states, I2, I2_XYZ, "0");
	connect(states, I2, I2_1, "1");
	connect(states, I2, I2_2, "2");
	connect(states, I2, I2_XY, "3-9");
	connect(states, I2_1, I2_XY, "0-9");
	connect(states, I2_2, I2_XY, "0-4");
	connect(states, I2_2, I2_25, "5");
	connect(states, I2_25, I2_XYZ, "0-5");
	connect(states, I2_XY, I2_XYZ, "0-9");
	connect(states, I2_1, I3, ".");
	connect(states, I2_2, I3, ".");
	connect(states, I2_25, I3, ".");
	connect(states, I2_XY, I3, ".");
	connect(states, I2_XYZ, I3, ".");
	connect(states, I2, T1, "A-Za-z");
	connect(states, I2, H1, "-!$&'()*+,;=");
	connect(states, I2_1, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I2_2, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I2_25, H1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, I2_XY, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I2_XYZ, H1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, I2, H2, "%");
	connect(states, I2_1, H2, "%");
	connect(states, I2_2, H2, "%");
	connect(states, I2_25, H2, "%");
	connect(states, I2_XY, H2, "%");
	connect(states, I2_XYZ, H2, "%");
	connect(states, I3, I3_XYZ, "0");
	connect(states, I3, I3_1, "1");
	connect(states, I3, I3_2, "2");
	connect(states, I3, I3_XY, "3-9");
	connect(states, I3_1, I3_XY, "0-9");
	connect(states, I3_2, I3_XY, "0-4");
	connect(states, I3_2, I3_25, "5");
	connect(states, I3_25, I3_XYZ, "0-5");
	connect(states, I3_XY, I3_XYZ, "0-9");
	connect(states, I3_1, I4, ".");
	connect(states, I3_2, I4, ".");
	connect(states, I3_25, I4, ".");
	connect(states, I3_XY, I4, ".");
	connect(states, I3_XYZ, I4, ".");
	connect(states, I3, T1, "A-Za-z");
	connect(states, I3, H1, "-!$&'()*+,;=");
	connect(states, I3_1, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I3_2, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I3_25, H1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, I3_XY, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I3_XYZ, H1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, I3, H2, "%");
	connect(states, I3_1, H2, "%");
	connect(states, I3_2, H2, "%");
	connect(states, I3_25, H2, "%");
	connect(states, I3_XY, H2, "%");
	connect(states, I3_XYZ, H2, "%");
	connect(states, I4, I4_XYZ, "0");
	connect(states, I4, I4_1, "1");
	connect(states, I4, I4_2, "2");
	connect(states, I4, I4_XY, "3-9");
	connect(states, I4_1, I4_XY, "0-9");
	connect(states, I4_2, I4_XY, "0-4");
	connect(states, I4_2, I4_25, "5");
	connect(states, I4_25, I4_XYZ, "0-5");
	connect(states, I4_XY, I4_XYZ, "0-9");
	connect(states, I4_1, P, ":");
	connect(states, I4_2, P, ":");
	connect(states, I4_25, P, ":");
	connect(states, I4_XY, P, ":");
	connect(states, I4_XYZ, P, ":");
	connect(states, I4_1, T0, ".");
	connect(states, I4_2, T0, ".");
	connect(states, I4_25, T0, ".");
	connect(states, I4_XY, T0, ".");
	connect(states, I4_XYZ, T0, ".");
	connect(states, I4, T1, "A-Za-z");
	connect(states, I4, H1, "-!$&'()*+,;=");
	connect(states, I4_1, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I4_2, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I4_25, H1, "-!$&'()*+,;=A-Za-z6-9");
	connect(states, I4_XY, H1, "-!$&'()*+,;=A-Za-z");
	connect(states, I4_XYZ, H1, "-!$&'()*+,;=A-Za-z0-9");
	connect(states, I4, H2, "%");
	connect(states, I4_1, H2, "%");
	connect(states, I4_2, H2, "%");
	connect(states, I4_25, H2, "%");
	connect(states, I4_XY, H2, "%");
	connect(states, I4_XYZ, H2, "%");
	connect(states, I4_1, A1, "/");
	connect(states, I4_2, A1, "/");
	connect(states, I4_25, A1, "/");
	connect(states, I4_XY, A1, "/");
	connect(states, I4_XYZ, A1, "/");
	connect(states, I4_1, Q1, "?");
	connect(states, I4_2, Q1, "?");
	connect(states, I4_25, Q1, "?");
	connect(states, I4_XY, Q1, "?");
	connect(states, I4_XYZ, Q1, "?");
	connect(states, I4_1, F1, "#");
	connect(states, I4_2, F1, "#");
	connect(states, I4_25, F1, "#");
	connect(states, I4_XY, F1, "#");
	connect(states, I4_XYZ, F1, "#");
	connect(states, H1, T0, ".");
	connect(states, H1, H1, "-_~A-Za-z0-9!$&'()*+,;=");
	connect(states, H1, H2, "%");
	connect(states, H2, H3, "0-9A-F");
	connect(states, H3, H1, "0-9A-F");
	connect(states, T1, T0, ".");
	connect(states, T2, T0, ".");
	connect(states, T3, T0, ".");
	connect(states, T4, T0, ".");
	connect(states, T5, T0, ".");
	connect(states, T6, T0, ".");
	connect(states, T0, T1, "A-Za-z");
	connect(states, T1, T2, "A-Za-z");
	connect(states, T2, T3, "A-Za-z");
	connect(states, T3, T4, "A-Za-z");
	connect(states, T4, T5, "A-Za-z");
	connect(states, T5, T6, "A-Za-z");
	connect(states, T0, H1, "-_~0-9!$&'()*+,;=");
	connect(states, T1, H1, "-_~0-9!$&'()*+,;=");
	connect(states, T2, H1, "-_~0-9!$&'()*+,;=");
	connect(states, T3, H1, "-_~0-9!$&'()*+,;=");
	connect(states, T4, H1, "-_~0-9!$&'()*+,;=");
	connect(states, T5, H1, "-_~0-9!$&'()*+,;=");
	connect(states, T6, H1, "-_~0-9!$&'()*+,;=A-Za-z");
	connect(states, T0, H2, "%");
	connect(states, T1, H2, "%");
	connect(states, T2, H2, "%");
	connect(states, T3, H2, "%");
	connect(states, T4, H2, "%");
	connect(states, T5, H2, "%");
	connect(states, T6, H2, "%");
	connect(states, T2, A1, "/");
	connect(states, T3, A1, "/");
	connect(states, T4, A1, "/");
	connect(states, T5, A1, "/");
	connect(states, T6, A1, "/");
	connect(states, T2, Q1, "?");
	connect(states, T3, Q1, "?");
	connect(states, T4, Q1, "?");
	connect(states, T5, Q1, "?");
	connect(states, T6, Q1, "?");
	connect(states, T2, F1, "#");
	connect(states, T3, F1, "#");
	connect(states, T4, F1, "#");
	connect(states, T5, F1, "#");
	connect(states, T6, F1, "#");
	connect(states, T2, P, ":");
	connect(states, T3, P, ":");
	connect(states, T4, P, ":");
	connect(states, T5, P, ":");
	connect(states, T6, P, ":");
	connect(states, P, P, "0-9");
	connect(states, P, A1, "/");
	connect(states, P, Q1, "?");
	connect(states, P, F1, "#");
	connect(states, A1, A2, "-._~!$&'()*+,;=A-Za-z0-9:@");
	connect(states, A2, A2, "-._~!$&'()*+,;=A-Za-z0-9:@");
	connect(states, A2, A1, "/");
	connect(states, A1, A3, "%");
	connect(states, A2, A3, "%");
	connect(states, A3, A4, "0-9A-F");
	connect(states, A4, A2, "0-9A-F");
	connect(states, A1, Q1, "?");
	connect(states, A1, F1, "#");
	connect(states, A2, Q1, "?");
	connect(states, A2, F1, "#");
	connect(states, Q1, Q1, "-._~!$&'()*+,;=A-Za-z0-9:@/?");
	connect(states, Q1, Q2, "%");
	connect(states, Q2, Q3, "0-9A-F");
	connect(states, Q3, Q1, "0-9A-F");
	connect(states, Q1, F1, "#");
	connect(states, F1, F1, "-._~!$&'()*+,;=A-Za-z0-9:@/?");
	connect(states, F1, F2, "%");
	connect(states, F2, F3, "0-9A-F");
	connect(states, F3, F1, "0-9A-F");
	return states;
}
